@@ -26,15 +26,11 @@ More info for the Python version is on Pypi: https://pypi.python.org/pypi/serpen
 [assembly: AssemblyTrademark("")]
 [assembly: AssemblyCulture("")]
 
-// This sets the default COM visibility of types in the assembly to invisible.
-// If you need to expose a type to COM, use [ComVisible(true)] on that type.
-[assembly: ComVisible(false)]
-
 // The assembly version has following format :
 //
 // Major.Minor.Build.Revision
 //
 // You can specify all the values or you can use the default the Revision and 
 // Build Numbers by using the '*' as shown below:
-[assembly: AssemblyVersion("1.12.0.*")]
-[assembly: AssemblyFileVersion("1.12.0.0")]
+[assembly: AssemblyVersion("1.16.0.*")]
+[assembly: AssemblyFileVersion("1.16.0.0")]
